## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
##  JNavalEnhancements by Jeckel.
##

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import Popup as PyPopup

import CvConfigParser
import CvPath

import JUtils
import CvJDataStorageUtils

######################################

# CvJDataStorageUtils.initTypesDict(pUnit, dictInit=None)
# CvJDataStorageUtils.initTypeStatsDict(pUnit, xType)
# CvJDataStorageUtils.hasTypesDict(pUnit)
# CvJDataStorageUtils.hasTypeStatsDict(pUnit, xType)
# CvJDataStorageUtils.hasTypeStat(pUnit, xType, xStat)
# CvJDataStorageUtils.setTypeStatsDict(pUnit, xType, dictStats=None)
# CvJDataStorageUtils.setTypeStat(pUnit, xType, xStat, xStatValue=None)
# CvJDataStorageUtils.delTypesDict(pUnit)
# CvJDataStorageUtils.delTypeStatsDict(pUnit, xType)
# CvJDataStorageUtils.delTypeStat(pUnit, xType, xStat)
# CvJDataStorageUtils.getDataIDStr(pObj=None)
# CvJDataStorageUtils.getUnitStatsStrType_StrStatList(pUnit, strDelim="_", bStatsTypeFirst=True)
# CvJDataStorageUtils.getTypesDict(pUnit)
# CvJDataStorageUtils.getTypeStatsDict(pUnit, xType)
# CvJDataStorageUtils.getTypeStat(pUnit, xType, xStat)



# JDataStorageUtils.initGlobalDict()
# JDataStorageUtils.hasGlobalDict()
# JDataStorageUtils.hasGlobalData(xKey)
# JDataStorageUtils.setGlobalData(xKey, xValue)
# JDataStorageUtils.getGlobalData(xKey=None)

###################################################

# globals
gc = CyGlobalContext()

############################################################################


# This option will turn the mod on or off.
# True - This will activate the JNavalEnhancements Mod.
# False - This will deactivate the JNavalEnhancements Mod.
# Default = True
g_bJNavalEnhancementsFeaturesEnabled = True


# How many times should a ship be allow to move its movement points
# worth of distance.
# Example: Galley's movement is 2.
#          This option is set to 3.
#          Galley's will be able to move 6 squares.
# This value must be a integer equal to or greater then 1.
# Anything less will default to 1.
# Default = 2
g_iNavalMovementMultiplier = 2


# Set the percent chance that ships can sink in Ocean tiles.
# This must be an integar equal to or greater then 0.
# Default = 50
g_iShipSinkBaseChance = 50


# With this option you can set an era to start applying a modifier to
# the base chance that a ship will sink.
# This option may be any valid game era.
# This option will be ignored if it is after the era set in
# the 'Last Sinkable Era' option.
# 	-ERA_ANCIENT
# 	-ERA_CLASSICAL
# 	-ERA_MEDIEVAL
# 	-ERA_RENAISSANCE
# 	-ERA_INDUSTRIAL
# 	-ERA_MODERN
# 	-ERA_FUTURE
# Default = ERA_CLASSICAL
g_strSinkChanceModifierStartEra= "ERA_CLASSICAL"


# Set the sinking ship chance modifier.
# This can be any positive or negative integar.
# Default = -25
g_iShipSinkChanceModifier = -25


# Set the last era that ships can randomly sink.
# Ships will sink in this era but not after it.
# 	-ERA_ANCIENT
# 	-ERA_CLASSICAL
# 	-ERA_MEDIEVAL
# 	-ERA_RENAISSANCE
# 	-ERA_INDUSTRIAL
# 	-ERA_MODERN
# 	-ERA_FUTURE
# Default = ERA_MEDIEVAL
g_strLastSinkableEra = "ERA_MEDIEVAL"

############################################################################


g_bDebug = False

def loadConfigurationFeaturesEnabledData():
	global g_bJNavalEnhancementsFeaturesEnabled

        # Load the JUnitReligion Mod Config INI file containing all of the configuration information
	config = CvConfigParser.CvConfigParser("JNavalEnhancements Mod Config.ini")
		
	# If the JUnitReligion Mod's config.ini file was found then load its values
	if (config != None):
                g_bJNavalEnhancementsFeaturesEnabled = config.getboolean("JNavalEnhancements Mod", "JNavalEnhancements Features Enabled", True)

def loadConfigurationData():
        global g_strSinkChanceModifierStartEra
        global g_strLastSinkableEra

	global g_iNavalMovementMultiplier
	global g_iShipSinkBaseChance
	global g_iShipSinkChanceModifier

        # Load the JUnitReligion Mod Config INI file containing all of the configuration information
	config = CvConfigParser.CvConfigParser("JNavalEnhancements Mod Config.ini")
		
	# If the JUnitReligion Mod's config.ini file was found then load its values
	if (config != None):
                g_strSinkChanceModifierStartEra = config.get("JNavalEnhancements Mod", "Sink Chance Modifier Start Era", "ERA_CLASSICAL")
                g_strLastSinkableEra = config.get("JNavalEnhancements Mod", "Last Sinkable Era", "ERA_MEDIEVAL")

                g_iNavalMovementMultiplier = config.getint("JNavalEnhancements Mod", "Naval Movement Multiplier", 2)
                if (g_iNavalMovementMultiplier < 1):
                        g_iNavalMovementMultiplier = 1

                g_iShipSinkBaseChance = config.getint("JNavalEnhancements Mod", "Ship Sink Base Chance", 50)
                if (g_iShipSinkBaseChance < 0):
                        g_iShipSinkBaseChance = 0

                if (g_iShipSinkBaseChance > 100):
                        g_iShipSinkBaseChance = 100

                g_iShipSinkChanceModifier = config.getint("JNavalEnhancements Mod", "Ship Sink Chance Modifier", -25)
                iMaxNegativeMod = g_iShipSinkBaseChance - (g_iShipSinkBaseChance * 2)
                if (g_iShipSinkChanceModifier < iMaxNegativeMod):
                        g_iShipSinkChanceModifier = iMaxNegativeMod

                iMaxMod = 100 - g_iShipSinkBaseChance
                if (g_iShipSinkChanceModifier > iMaxMod):
                        g_iShipSinkChanceModifier = iMaxMod



class JNavalEnhancements:

        ###################################################################
        # These methods produce messages in the log and/or on the screen. #
        ###################################################################

        def alertPlayer(self, iPlayer, message, iColor = 7):

                pPlayer = gc.getPlayer(iPlayer)

                # Return immediately if the player passed in is invalid
		if (pPlayer == None) or (pPlayer.isNone()) or (not pPlayer.isHuman()):
			return None

                eventMessageTimeLong = gc.getDefineINT("EVENT_MESSAGE_TIME_LONG")
                szIcon = None
                iFlashX = 0
                iFlashY = 0

                szString = str(message)

                CyInterface().addMessage(iPlayer, True, eventMessageTimeLong,
                                         szString, None, 0, szIcon, ColorTypes(iColor),
                                         iFlashX, iFlashY, False, False)

        def alertIconPlayer(self, iPlayer, message, pObject, szIcon = None, iColor = 7):

                pPlayer = gc.getPlayer(iPlayer)

                # Return immediately if the player passed in is invalid
		if (pPlayer == None) or (pPlayer.isNone()) or (not pPlayer.isHuman()):
			return None

                # Return immediately if the object passed in is invalid
		if (pObject == None) or (pObject.isNone()):
			return None

                eventMessageTimeLong = gc.getDefineINT("EVENT_MESSAGE_TIME_LONG")

                strMessage = str(message)

                CyInterface().addMessage(iPlayer, True, eventMessageTimeLong,
                                         strMessage, None, 0, szIcon, ColorTypes(iColor),
                                         pObject.getX(), pObject.getY(), True, True)

        #################################################
        # These are get methods that return some value. #
        #################################################

        def getShipTimesMoved(self, pUnit):

                # Return immediately if the unit passed in is invalid
		if (pUnit == None) or (pUnit.isNone()):
			return 0

		# Does the unit have stored data initilezed, if not then do it
                if (not self.hasShipData(pUnit)):
                        self.setupShipData(pUnit)

                return CvJDataStorageUtils.getTypeStat(pUnit, "NavalData", "iTimesMoved")

        ##########################################################################################
        # These are the has methods. They take some form of input and return bool True or False. #
        ##########################################################################################

        

        ##########################################################################################
        # These are the is methods. They take some form of input and return bool True or False.  #
        ##########################################################################################

        

        #########################################################
        # These methods deal with data storage using SdToolKit. #
        #########################################################

        def setupShipData(self, pUnit):

                # Return immediately if the unit passed in is invalid
		if (pUnit == None) or (pUnit.isNone()):
			return None

		CvJDataStorageUtils.initTypeStatsDict(pUnit, "NavalData")
		CvJDataStorageUtils.setTypeStat(pUnit, "NavalData", "iTimesMoved", 0)

        def hasShipData(self, pUnit):

                # Return immediately if the unit passed in is invalid
		if (pUnit == None) or (pUnit.isNone()):
			return False

		return CvJDataStorageUtils.hasTypeStatsDict(pUnit, "NavalData")


        def clearShipData(self, pUnit):

                # Return immediately if the unit passed in is invalid
		if (pUnit == None) or (pUnit.isNone()):
			return None

		return CvJDataStorageUtils.delTypeStatsDict(pUnit, "NavalData")


        def resetShipTimesMoved(self, pUnit):

                # Return immediately if the unit passed in is invalid
		if (pUnit == None) or (pUnit.isNone()):
			return None

                # Does the unit have stored data initilezed, if not then do it
                if (not self.hasShipData(pUnit)):
                        self.setupShipData(pUnit)

                return CvJDataStorageUtils.setTypeStat(pUnit, "NavalData", "iTimesMoved", 0)


        def storeShipTimesMovedPlusOne(self, pUnit):

                # Return immediately if the unit passed in is invalid
		if (pUnit == None) or (pUnit.isNone()):
			return None

                # Does the unit have stored data initilezed, if not then do it
                if (not self.hasShipData(pUnit)):
                        self.setupShipData(pUnit)

                strShipID = str(pUnit.getOwner()) + "-" + str(pUnit.getID())

                iOldValue = CvJDataStorageUtils.getTypeStat(pUnit, "NavalData", "iTimesMoved")
                iNewValue = iOldValue + 1

		return CvJDataStorageUtils.setTypeStat(pUnit, "NavalData", "iTimesMoved", iNewValue)


	

        ########################################################
        # These are the set methods and don't return anything. #
        ########################################################

        

        ########################################################
        # These are the run methods and don't return anything. #
        ########################################################

        


        ##########################################
        # These are the onEvent Handler methods. #
        ##########################################

	def onUnitMove(self, argsList):
		'unit move'
		pPlot, pUnit = argsList

                iPlayer = pUnit.getOwner()
		pPlayer = gc.getPlayer(iPlayer)
		iTeam = pPlayer.getTeam()
		pTeam = gc.getTeam(iTeam)

                iUnitDomain = pUnit.getDomainType()
                iSeaDomain = gc.getInfoTypeForString("DOMAIN_SEA")

                if (iUnitDomain != iSeaDomain):
                        return None

                loadConfigurationData()

                iRand = CyGame().getSorenRandNum(101, "Random Ship Sinking")

                iPlayerEra = pPlayer.getCurrentEra()
                iLastSinkableEra = gc.getInfoTypeForString(g_strLastSinkableEra)
                iStartSinkModEra = gc.getInfoTypeForString(g_strSinkChanceModifierStartEra)

                iSinkChance = -1
                if (iPlayerEra <= iLastSinkableEra) and (pPlot.getTerrainType() == gc.getInfoTypeForString("TERRAIN_OCEAN")):
                        iSinkChance = g_iShipSinkBaseChance
                        if (iPlayerEra >= iStartSinkModEra):
                                iSinkChance = g_iShipSinkBaseChance + g_iShipSinkChanceModifier

                JUtils.alertDebug(0, "iSinkChance", iSinkChance)
                JUtils.alertDebug(0, "iRand", iRand)
                if (iRand <= iSinkChance):
                        pUnit.kill(True, iPlayer)
                        JUtils.alert(iPlayer, "Your ship has been lost at sea.")
                        return None

                iX = pUnit.getX()
                iY = pUnit.getY()

                iEnemyNavalZoCCount = 0
                for iiX in range(iX-2, iX+3):
                        for iiY in range(iY-2, iY+3):
                                pPlot = CyMap().plot(iiX,iiY)
                                for iTempUnit in range(pPlot.getNumUnits()):
                                        pTempUnit = pPlot.getUnit(iTempUnit)
                                        iTempPlayer = pTempUnit.getOwner()
                                        if (iPlayer == iTempPlayer):
                                                continue
                                        pTempPlayer = gc.getPlayer(iTempPlayer)
                                        iTempTeam = pTempPlayer.getTeam()
                                        if (pTeam.isAtWar(iTempTeam)):
                                                iEnemyNavalZoCCount += 1
                                                break
                                        continue

                JUtils.alertDebug(0, "iEnemyNavalZoCCount", iEnemyNavalZoCCount)
                if (iEnemyNavalZoCCount >= 1):
                        pUnit.finishMoves()
                        

                iMovesLeft = pUnit.movesLeft()

                # If the unit still has movement then return None
                if (iMovesLeft > 0):
                        return None

                JUtils.alertDebug(0, "iMovesLeft", iMovesLeft)
                JUtils.alertDebug(0, "self.getShipTimesMoved(pUnit)1", self.getShipTimesMoved(pUnit))

                self.storeShipTimesMovedPlusOne(pUnit)

                JUtils.alertDebug(0, "self.getShipTimesMoved(pUnit)2", self.getShipTimesMoved(pUnit))
                JUtils.alertDebug(0, "g_iNavalMovementMultiplier", g_iNavalMovementMultiplier)

                if (g_iNavalMovementMultiplier <= self.getShipTimesMoved(pUnit)):
                        return None

                iMaxMoves = pUnit.maxMoves()
                pUnit.setMoves(0)


        def onEndGameTurn(self, argsList):
		'Called at the end of the end of each turn'
		iGameTurn = argsList[0]

		iTotalPlayers = gc.getMAX_PLAYERS()

		for iPlayer in range(iTotalPlayers):
                        pPlayer = gc.getPlayer(iPlayer)

                        # Return immediately if the player passed in is invalid
                        if (pPlayer == None) or (pPlayer.isNone()) or (not pPlayer.isAlive()):
                                continue
			
                        iTotalUnits = pPlayer.getNumUnits()

                        for iUnit in range(iTotalUnits):
                                pUnit = pPlayer.getUnit(iUnit)

                                # Return immediately if the unit passed in is invalid
                                if (pUnit == None) or (pUnit.isNone()) or (pUnit.isDead()):
                                        continue

                                iUnitDomain = pUnit.getDomainType()
                                iSeaDomain = gc.getInfoTypeForString("DOMAIN_SEA")

                                if (iUnitDomain != iSeaDomain):
                                        continue

                                #iMovesLeft = pUnit.movesLeft()
                                #iMaxMoves = pUnit.maxMoves()

                                #iMovesModifier = iMovesLeft + iMaxMoves

                                #iNewMoves = iMovesLeft - iMovesModifier

                                self.resetShipTimesMoved(pUnit)


	def onUnitCreated(self, argsList):
		'Unit Completed'
		pUnit = argsList[0]
		iPlayer = pUnit.getOwner()
		pPlayer = gc.getPlayer(iPlayer)

		iUnitDomain = pUnit.getDomainType()
                iSeaDomain = gc.getInfoTypeForString("DOMAIN_SEA")

                if (iUnitDomain == iSeaDomain):
                        self.setupShipData(pUnit)


	def onUnitLost(self, argsList):
		'Unit Lost'
		pUnit = argsList[0]
		iPlayer = pUnit.getOwner()
		pPlayer = gc.getPlayer(iPlayer)

		iUnitDomain = pUnit.getDomainType()
                iSeaDomain = gc.getInfoTypeForString("DOMAIN_SEA")

                if (iUnitDomain == iSeaDomain):
                        self.clearShipData(pUnit)
















